#  Scraper for Kansas Appeals Court
# CourtID: kan_u


from juriscraper.opinions.united_states.state import kan_p


class Site(kan_p.Site):
    def __init__(self, *args, **kwargs):
        super(Site, self).__init__(*args, **kwargs)
        self.court_id = self.__module__
        self.status = "Unpublished"
